#include "mbed.h"
#include "lis3dh.h"


// Defines ////////////////////////////////////////////////////////////////

// The Arduino two-wire interface uses a 7-bit number for the address, 
// and sets the last bit correctly based on reads and writes
// mbed I2C libraries take the 7-bit address shifted left 1 bit
// #define ACC_ADDRESS (0x30 >> 1)

//SA0 = heigh ---> 0x31
//SA0 = low   ---> 0x30
#define ACC_ADDRESS 0x30


// Public Methods //////////////////////////////////////////////////////////////

// Constructor
LIS3DH::LIS3DH(PinName sda, PinName scl):
    _LIS3DH(sda, scl)
{
    char reg_v;
    _LIS3DH.frequency(100000);
    
  // 0x0F = 0b00001111
  // Normal power mode, all axes enabled
    reg_v = 0;    
    reg_v |= 0x7F;       
    write_reg(ACC_ADDRESS,LIS3DH_CTRL_REG1,reg_v);
    


}



char LIS3DH::whoami(void)
{
char work_c[1];

    read_reg(ACC_ADDRESS, LIS3DH_WHO_AM_I, work_c);
    return(work_c[0]);
}

bool LIS3DH::read(float *gx, float *gy, float *gz) {
    char gyr[6];
 
    if (recv(ACC_ADDRESS, LIS3DH_OUT_X_L, gyr, 6)) {
    //scale is +-2g
        *gx = float(short(gyr[1] << 8 | gyr[0]))*6.10361e-5;  
        *gy =  float(short(gyr[3] << 8 | gyr[2]))*6.10361e-5;
        *gz =  float(short(gyr[5] << 8 | gyr[4]))*6.10361e-5;

 
        return true;
    }
 
    return false;
}




bool LIS3DH::write_reg(int addr_i2c,int addr_reg, char v)
{
    char data[2] = {addr_reg, v}; 
    return LIS3DH::_LIS3DH.write(addr_i2c, data, 2) == 0;
}

bool LIS3DH::read_reg(int addr_i2c,int addr_reg, char *v)
{
    char data = addr_reg; 
    bool result = false;
    
    __disable_irq();
    if ((_LIS3DH.write(addr_i2c, &data, 1) == 0) && (_LIS3DH.read(addr_i2c, &data, 1) == 0)){
        *v = data;
        result = true;
    }
    __enable_irq();
    return result;
}


bool LIS3DH::recv(char sad, char sub, char *buf, int length) {
    if (length > 1) sub |= 0x80;
 
    return _LIS3DH.write(sad, &sub, 1, true) == 0 && _LIS3DH.read(sad, buf, length) == 0;
}
