#ifndef __LIS3DH_H
#define __LIS3DH_H

#include "mbed.h"

// register addresses

#define LIS3DH_WHO_AM_I      0x0F

#define LIS3DH_CTRL_REG1     0x20
#define LIS3DH_CTRL_REG2     0x21
#define LIS3DH_CTRL_REG3     0x22
#define LIS3DH_CTRL_REG4     0x23
#define LIS3DH_CTRL_REG5     0x24
#define LIS3DH_REFERENCE     0x25
#define LIS3DH_OUT_TEMP      0x26
#define LIS3DH_STATUS_REG    0x27

#define LIS3DH_OUT_X_L       0x28
#define LIS3DH_OUT_X_H       0x29
#define LIS3DH_OUT_Y_L       0x2A
#define LIS3DH_OUT_Y_H       0x2B
#define LIS3DH_OUT_Z_L       0x2C
#define LIS3DH_OUT_Z_H       0x2D

#define LIS3DH_FIFO_CTRL_REG 0x2E
#define LIS3DH_FIFO_SRC_REG  0x2F

#define LIS3DH_INT1_CFG      0x30
#define LIS3DH_INT1_SRC      0x31
#define LIS3DH_INT1_THS_XH   0x32
#define LIS3DH_INT1_THS_XL   0x33
#define LIS3DH_INT1_THS_YH   0x34
#define LIS3DH_INT1_THS_YL   0x35
#define LIS3DH_INT1_THS_ZH   0x36
#define LIS3DH_INT1_THS_ZL   0x37
#define LIS3DH_INT1_DURATION 0x38

/** Interface library for the ST LIS3DH 3-axis accerelation sensor
 *
 *
 * @code
 * #include "mbed.h"
 * #include "LIS3DH.h"
 * LIS3DH gyro(p28, p27);
 * ...
 * int g[3];
 * gyro.read(g);
 * @endcode
 */
class LIS3DH
{
    public:
        /** Create a new LIS3DH I2C interface
         * @param sda is the pin for the I2C SDA line
         * @param scl is the pin for the I2C SCL line
         */
        LIS3DH(PinName sda, PinName scl);
        
        /** Read gyro values
         * @param a Array containing x, y, and z acc. values
         * @return a Array containing x, y, and z acc. values
         */
        char whoami(void);
        bool read(float *ax, float *ay, float *az);
        
    private:
            I2C _LIS3DH;
        float ax, ay, az;

        bool write_reg(int addr_i2c,int addr_reg, char v);
        bool read_reg(int addr_i2c,int addr_reg, char *v);
        bool recv(char sad, char sub, char *buf, int length);
};

#endif