#include "mbed.h"
#include "SDFileSystem.h"
#include "L3GD20.h"
#include "lis3dh.h"
#include <stdio.h>
#include "2l5v1a.h"

SDFileSystem sd(p5, p6, p7, p8, "sd");

DigitalOut led1(LED1);
DigitalOut led2(LED2);
DigitalOut led3(LED3);
DigitalOut led4(LED4);

PwmOut buzzer(p26);

//Tristate buffer switching port
//Right body or whole body joints
DigitalOut gr1_tx(p17);
DigitalOut gr1_rx(p18);

//Left body joints or Xbee comm. port
DigitalOut gr2_tx(p19);
DigitalOut gr2_rx(p20);

DigitalIn dip_sw1(p11);
DigitalIn dip_sw2(p12);

AnalogIn Battery(p15);

Serial pc(USBTX, USBRX); // tx, rx 9600bps
//Serial servo_L(p13, p14);
Serial xbee(p13, p14); // tx, rx 9600bps default, Port connected to Xbee s2
Serial servo_R(p9, p10);

L3GD20 l3gd20(p28, p27);//gyro.
LIS3DH lis3dh(p28, p27);//accel.

Timer timer;

float offset_ax, offset_ay, offset_az;
float offset_gx, offset_gy, offset_gz;

int main()
{
    unsigned char command;
    int     num, dip_sw;
    int     i, j, data, id;
    unsigned int return_data[6];
    float gx, gy, gz, input_volt;
    float ax, ay, az;

    struct SIO sio;

    for(i = 0; i < 320; i++) {
        for(j = 0; j < 6; j++) {
            sio.motion_log[i][j] = 0.0;
        }
    }
    sio.motion_log_flg = 0;
    sio.motion_log_datanum = 0;

    dip_sw = 0;
    //dip_sw1&2 is active low.
    if (dip_sw1 == 0) dip_sw = dip_sw + 1;
    if (dip_sw2 == 0) dip_sw = dip_sw + 2;

    //pc.baud(460800);
    pc.baud(921600);
    //pc.baud(115200);

    xbee.baud(57600);
//   xbee.printf("This is a test message from xbee connection!\n\n");

//    servo_L.baud(1250000);
//    servo_L.format(8, SerialBase::Even, 1);

    servo_R.baud(1250000);
    servo_R.format(8, SerialBase::Even, 1);

    gr1_tx = 1;
    gr1_rx = 1;

    gr2_tx = 1;
    gr2_rx = 1;

    input_volt = Battery * 3.3 * 6.0;

    //Sensor newtral value acquisition start.
    buzzer.period_us(700);
    buzzer = 0.5;
    wait_ms(70);
    buzzer = 0.0;
    wait_ms(70);
    buzzer = 0.5;
    wait_ms(70);
    buzzer = 0.0;
    //wait_ms(180);

    //Init accel LIS3DH
    i = (int)lis3dh.whoami();
    if(dip_sw == 0) {
        printf("lis3dh whoami : %d", i);
        if(i == 51) printf(" --> OK\n");
        else printf(" --> NG\n");
    }

    offset_ax = 0.0;
    offset_ay = 0.0;
    offset_az = -1.0;

    sio.gain_gx = 0.0;
    sio.gain_gy = 0.0;
    sio.gain_gz = 0.0;

    if(i == 51) {
        wait(1);
        for(i = 0; i < 10; i++) {
            lis3dh.read(&ax, &ay, &az);
            offset_ax = offset_ax + ax;
            offset_ay = offset_ay + ay;
            offset_az = offset_az + az;
            wait(0.1);
        }
        offset_ax = offset_ax / 10.0;
        offset_ay = offset_ay / 10.0;
        offset_az = offset_az / 10.0 - 1.0;
    }

    //Init gyro
    i = (int)l3gd20.whoami();
    if(dip_sw == 0) {
        printf("l3gd20 whoami : %d", i);
        if(i == 212) printf(" --> OK\n");
        else printf(" --> NG\n");
    }

    offset_gx = 0.0;
    offset_gy = 0.0;
    offset_gz = 0.0;
    if(i == 212) {
        for(i = 0; i < 10; i++) {
            l3gd20.read(&gx, &gy, &gz);
            offset_gx = offset_gx + gx;
            offset_gy = offset_gy + gy;
            offset_gz = offset_gz + gz;
            wait(0.1);
        }
        offset_gx = offset_gx / 10.0;
        offset_gy = offset_gy / 10.0;
        offset_gz = offset_gz / 10.0;
    }

    if(input_volt > 9.0) {
        //OK
        buzzer.period_us(700);
    } else {
        //NG
        buzzer.period_us(12000);
    }
    buzzer = 0.5;
    wait_ms(500);
    buzzer = 0.0;

    if(dip_sw == 0) {
        pc.printf("Input voltage = %5.1f\r\n",input_volt);
        pc.printf("\n****** 2l5v1a start! ******\n");
        pc.printf("DIP SW = %d\n", dip_sw);
    }

    switch(dip_sw) {
        case 0:
            //作業選択メニュー表示
            num = 0;
            while (!pc.writeable());
            while(num != 99) {
                printf("\n");
                printf("Select number.\n");
                printf("\n");
                printf(" 1:Single Servo move test..\n");
                printf("\n");
                printf(" 2:Servo parameter read\n");
                printf("\n");
                printf(" 3:Single servo move test 6000-9000\n");
                printf("\n");
                printf(" 4:Show L3GD20(gyro) status.\n");
                printf("\n");
                printf(" 5:Show LIS3DH(accel) status.\n");
                printf("\n");
                printf(" 6:Set gyro gain value.\n");
                printf("\n");
                printf(" 7:gyro feed back testing.(move id9 & id20 servo)\n");
                printf("\n");
                printf("99:Exit loop\n\n");

                //作業番号をキーボードから読み取る
                scanf ("%d",&num);
                while (!pc.readable());
                i = pc.getc();//dummy LF read
                printf("%d\n", num);
                /*******************************************************/
                //Single Servo move test
                if(num == 1) {
                    printf("Enter servo id\n");
                    scanf ("%d",&id);
                    while (!pc.readable());
                    i = pc.getc();//dummy LF read
                    printf("%d\n", id);

                    printf("Enter output data\n");
                    scanf ("%d",&data);
                    while (!pc.readable());
                    i = pc.getc();//dummy LF read
                    printf("%d\n", data);

                    move_servo(&servo_R, id, data, &return_data[0]);
                    /*
                    for(i = 0; i < 3; i++) {
                        printf("%x\n", return_data[i]);
                    }
                    */
                    printf("Return data : %6d\n", return_data[0]);
                }
                /*******************************************************/
                //Servo parameter read
                if(num == 2) {
                    printf("\n");
                    printf("Input subcommand.(Ex. 1)\n");
                    scanf ("%d",&data);
                    while (!pc.readable());
                    i = pc.getc();//dummy LF read
                    printf("%d\n", data);

                    read_servo(&servo_R, 4, data, return_data);
                    for(i = 0; i < 3; i++) {
                        printf("%x\n", return_data[i]);
                    }
                }
                /*******************************************************/
                //Single Servo move test
                if(num == 3) {
                    data = 7500;
                    for(i = 0; i < 300; i++) {
                        move_servo(&servo_R, 4, data, return_data);
                        data = data - 3;
                        wait_ms(5);
                    }
                    for(i = 0; i < 300; i++) {
                        move_servo(&servo_R, 4, data, return_data);
                        data = data + 6;
                        wait_ms(5);
                    }
                    for(i = 0; i < 300; i++) {
                        move_servo(&servo_R, 4, data, return_data);
                        data = data - 3;
                        wait_ms(10);
                    }
                }
                /*******************************************************/
                //Show GYRO status
                if(num == 4) {
                    l3gd20.read(&gx, &gy, &gz);
                    gx = gx - offset_gx;
                    gy = gy - offset_gy;
                    gz = gz - offset_gz;
                    printf("L3GD20 : gx = %6.1f, gy = %6.1f, gz = %6.1f\n", gx, gy, gz);
                }
                /*******************************************************/
                //Show accel. status
                if(num == 5) {
                    lis3dh.read(&ax, &ay, &az);
                    ax = ax - offset_ax;
                    ay = ay - offset_ay;
                    az = az - offset_az;
                    printf("LIS3DH : ax = %6.3f, ay = %6.3f, az = %6.3f\n", ax, ay, az);
                }
                /*******************************************************/
                //Get gyro gain value.
                if(num == 6) {
                    printf("\n");
                    printf("Input gyro gain value for pitch axis(gx).(ex. 0.1)\n");
                    scanf ("%f",&sio.gain_gx);

                    printf("\n");
                    printf("%f\n", sio.gain_gx);

                    sio.gain_gy = sio.gain_gx;
                }
                /*******************************************************/
                //Get feedback testing.
                if(num == 7) {
                    sio.gain_gx = 1.0;
                    sio.gain_gy = 1.0;
                    sio.gain_gz = 1.0;

                    printf("\nGyro gain gx : %f\n", sio.gain_gx);
                    printf("This function will continue 5 second.\n");
                    for(i = 0; i < 500; i++) {
                        timer.reset();
                        timer.start();

                        l3gd20.read(&gx, &gy, &gz);
                        gx = gx - offset_gx;
                        gy = gy - offset_gy;
                        sio.servo_data[9]  = 6875 + (int)(sio.gain_gx * gx);
                        sio.servo_data[10]  = 7500 + (int)(sio.gain_gy * gy);

                        sio.servo_data[20] = 8125 - (int)(sio.gain_gx * gx);
                        sio.servo_data[21]  = 7450 + (int)(sio.gain_gy * gy);

                        sio.servo_data[11]  = 7500 + (int)(sio.gain_gz * gz);//body

                        move_servo(&servo_R, 9,  sio.servo_data[9], &return_data[0]);
                        move_servo(&servo_R,10, sio.servo_data[10], &return_data[0]);
                        move_servo(&servo_R,20, sio.servo_data[20], &return_data[0]);
                        move_servo(&servo_R,21, sio.servo_data[21], &return_data[0]);

                        move_servo(&servo_R,11, sio.servo_data[11], &return_data[0]);

                        //wait_ms(1);
                        while(timer.read_us()<10000);
                    }
                    move_servo(&servo_R, 9, 6875, &return_data[0]);
                    move_servo(&servo_R,10, 7500, &return_data[0]);
                    move_servo(&servo_R,20, 8125, &return_data[0]);
                    move_servo(&servo_R,21, 7450, &return_data[0]);
                }
            }
        case 1:

        default:
            if(dip_sw == 0) pc.printf("Ready to get command from 2L5v1_vb.\r\n");
            while(1) {
                timer.reset();
                timer.start();
                command = 0;
                if (pc.readable()) {
                    command = pc.getc();
                    sio_comm1(command, &sio);
                }
                //wait_ms(1);
                while(timer.read_us()<10000);
            }
    }
}




