#include "mbed.h"
#include "L3GD20.h"
#include "lis3dh.h"
#include "2l5v1a.h"

extern DigitalOut led1;
extern DigitalOut led2;
extern DigitalOut led3;
extern DigitalOut led4;

extern AnalogIn Battery;

extern Serial pc;
//extern Serial servo_L;
extern Serial servo_R;

extern L3GD20 l3gd20;
extern LIS3DH lis3dh;

extern PwmOut buzzer;

extern float offset_ax, offset_ay, offset_az;
extern float offset_gx, offset_gy, offset_gz;

//*******************************************
//*******************************************
void sio_comm1(unsigned char command, struct SIO *sio)
{
    unsigned char sub_command;
    int i, j, id, id_L, id_R;
    unsigned int ii, return_data;
    float gx, gy, gz;
    float ax, ay, az;

    //Send back ack
    while (!pc.writeable());
    pc.putc(command);

    //Receive all servo data
    //(return position data)
    if (command == 'A') {
        //servo data receive from PC
        i = 0;
        for (id = 0; id < 22; id++) {
            sio->servo_data[id] = read_ii(&pc);
            if(i == 5) {
                //Return byte to regurate data send interval.
                pc.putc(command);
                i = 0;
            }
            i++;
        }

        //servo data send to servo
        for (id = 0; id < 22; id++) {
            move_servo(&servo_R, id, sio->servo_data[id], &return_data);
            sio->servo_data_ret[id] = return_data;
        }

        //servo data send to PC
        for (id = 0; id < 22; id++) {
            print_ii(&pc, sio->servo_data_ret[id]);
        }
    }

    //Receive all servo data
    //(return NO position data)
    if (command == 'B') {
        //servo data receive from PC
        i = 0;
        for (id = 0; id < 22; id++) {
            sio->servo_data[id] = read_ii(&pc);
            if(i == 6) {
                //Return byte to regurate data send interval.
                pc.putc(command);
                i = 0;
            }
            i++;
        }

        //gyro feedback
        if(sio->gain_gx != 0.0 || sio->gain_gy != 0.0) {
            l3gd20.read(&gx, &gy, &gz);
            gx = gx - offset_gx;
            gy = gy - offset_gy;

            //ankle pitch axis
            sio->servo_data[9]  = sio->servo_data[9] + (int)(sio->gain_gx * gx);
            sio->servo_data[20] = sio->servo_data[20] - (int)(sio->gain_gx * gx);
            //ankle roll axis
            sio->servo_data[10]  = sio->servo_data[10] + (int)(sio->gain_gy * gy);
            sio->servo_data[21]  = sio->servo_data[21] + (int)(sio->gain_gy * gy);
        }

        //servo data send to servo
        id_R = 11;
        for (id_L = 0; id_L < 11; id_L++) {
            move_servo(&servo_R, id_L, sio->servo_data[id_L], &return_data);
            move_servo(&servo_R, id_R, sio->servo_data[id_R], &return_data);
            id_R++;
        }
    }

    //Receive one servo data
    //(return position data)
    if (command == 'C') {
        //servo id receive from PC
        id = pc.getc();

        //servo data receive from PC
        sio->servo_data[id] = read_ii(&pc);

        //servo data send to servo
        move_servo(&servo_R, id, sio->servo_data[id], &return_data);

        //servo data send to PC
        print_ii(&pc, return_data);
    }

    //gyro. report
    if (command == 'D') {
        //adjust range and convert into integer the return value
        //range +-250.0 degree per sec.(16bit)
        l3gd20.read(&gx, &gy, &gz);
        return_data = (unsigned int)((gx + 250.0) * 131.0);
        print_ii(&pc, return_data);

        return_data = (unsigned int)((gy + 250.0) * 131.0);
        print_ii(&pc, return_data);

        return_data = (unsigned int)((gz + 250.0) * 131.0);
        print_ii(&pc, return_data);
    }

    //accel. report
    if (command == 'E') {
        //adjust range and convert into integer the return value
        //range +-2.0 g.(12bit)
        lis3dh.read(&ax, &ay, &az);
        return_data = (unsigned int)((ax + 2.0) * 1023.0);
        print_ii(&pc, return_data);

        return_data = (unsigned int)((ay + 2.0) * 1023.0);
        print_ii(&pc, return_data);

        return_data = (unsigned int)((az + 2.0) * 1023.0);
        print_ii(&pc, return_data);
    }

    //get gyro gain for internal feedback use.
    if (command == 'F') {
        //get the value in integer and x100 multiple
        sio->gain_gx = (float)read_ii(&pc) / 100.0;
        sio->gain_gy = (float)read_ii(&pc) / 100.0;
        sio->gain_gz = (float)read_ii(&pc) / 100.0;
    }

    //get gyro neutral point
    if (command == 'G') {
        buzzer.period_us(700);
        buzzer = 0.5;
        wait_ms(70);
        buzzer = 0.0;

        offset_gx = 0.0;
        offset_gy = 0.0;
        offset_gz = 0.0;

        offset_ax = 0.0;
        offset_ay = 0.0;
        offset_az = 0.0;

        for(i = 0; i < 30; i++) {
            l3gd20.read(&gx, &gy, &gz);
            offset_gx = offset_gx + gx;
            offset_gy = offset_gy + gy;
            offset_gz = offset_gz + gz;

            lis3dh.read(&ax, &ay, &az);
            offset_ax = offset_ax + ax;
            offset_ay = offset_ay + ay;
            offset_az = offset_az + az;
            wait(0.1);
        }
        offset_gx = offset_gx / 30.0;
        offset_gy = offset_gy / 30.0;
        offset_gz = offset_gz / 30.0;

        offset_ax = offset_ax / 30.0;
        offset_ay = offset_ay / 30.0;
        offset_az = offset_az / 30.0 - 1.0;

        buzzer = 0.5;
        wait_ms(70);
        buzzer = 0.0;
    }

    //Receive all servo data with a switch byte
    //And Back gyro & acc. data
    //(return NO position data)
    if (command == 'a') {
        //servo data receive from PC
        i = 0;
        for (id = 0; id < 22; id++) {
            sio->servo_data[id] = read_ii(&pc);
            if(i == 6) {
                //Return byte to regurate data send interval.
                pc.putc(command);
                i = 0;
            }
            i++;
        }
        //gyro switch byte data receive from PC
        ii = read_ii(&pc);
        sio->switches = (unsigned char)ii;

        //gyro feedback
        l3gd20.read(&gx, &gy, &gz);
        gx = gx - offset_gx;
        gy = gy - offset_gy;
        gz = gz - offset_gz;

        if(sio->gain_gx != 0.0 || sio->gain_gy != 0.0) {
            if((sio->switches & 0x01) > 0) {
                //ankle pitch axis
                sio->servo_data[9]  = sio->servo_data[9] + (int)(sio->gain_gx * gx);
                //ankle roll axis
                sio->servo_data[10]  = sio->servo_data[10] + (int)(sio->gain_gy * gy);
            }

            if((sio->switches & 0x02) > 0) {
                //ankle pitch axis
                sio->servo_data[20] = sio->servo_data[20] - (int)(sio->gain_gx * gx);
                //ankle roll axis
                sio->servo_data[21]  = sio->servo_data[21] + (int)(sio->gain_gy * gy);
            }

            //body yaw axis
            sio->servo_data[11]  = sio->servo_data[11] + (int)(sio->gain_gz * gz);
        }

        //servo data send to servo
        id_R = 11;
        for (id_L = 0; id_L < 11; id_L++) {
            move_servo(&servo_R, id_L, sio->servo_data[id_L], &return_data);
            move_servo(&servo_R, id_R, sio->servo_data[id_R], &return_data);
            id_R++;
        }

        if(sio->motion_log_flg == 1) {
            //gyro. report
            //adjust range and convert into integer the return value
            //range +-250.0 degree per sec.(16bit)
            //Use the same data as the "gyro feed back". See sveral lines before.
            sio->motion_log[sio->motion_log_datanum][0] = gx;
            sio->motion_log[sio->motion_log_datanum][1] = gy;
            sio->motion_log[sio->motion_log_datanum][2] = gz;

            //accel. report
            //adjust range and convert into integer the return value
            //range +-2.0 g.(12bit)
            if(lis3dh.read(&ax, &ay, &az) == true) {
                ax = ax - offset_ax;
                ay = ay - offset_ay;
                az = az - offset_az;

                sio->motion_log[sio->motion_log_datanum][3] = ax;
                sio->motion_log[sio->motion_log_datanum][4] = ay;
                sio->motion_log[sio->motion_log_datanum][5] = az;
            }

            if(sio->motion_log_datanum < 320) {
                sio->motion_log_datanum++;
            } else {
                sio->motion_log_datanum = 0;
            }
        }
    }

    //Receive all servo data with a switch byte
    //(return NO position data)
    if (command == 'b') {
        //servo data receive from PC
        i = 0;
        for (id = 0; id < 22; id++) {
            sio->servo_data[id] = read_ii(&pc);
            if(i == 6) {
                //Return byte to regurate data send interval.
                pc.putc(command);
                i = 0;
            }
            i++;
        }
        //switch byte data receive from PC
        ii = read_ii(&pc);
        sio->switches = (unsigned char)ii;

        //gyro feedback
        if(sio->gain_gx != 0.0 || sio->gain_gy != 0.0) {
            l3gd20.read(&gx, &gy, &gz);
            gx = gx - offset_gx;
            gy = gy - offset_gy;

            if((sio->switches & 0x01) > 0) {
                //hip roll axis
                //sio->servo_data[6]  = sio->servo_data[6] + (int)(sio->gain_gy * gy * 0.5);
                //ankle pitch axis
                sio->servo_data[9]  = sio->servo_data[9] + (int)(sio->gain_gx * gx);
                //ankle roll axis
                sio->servo_data[10]  = sio->servo_data[10] + (int)(sio->gain_gy * gy);
            }

            if((sio->switches & 0x02) > 0) {
                //hip roll axis
                //sio->servo_data[17]  = sio->servo_data[17] + (int)(sio->gain_gy * gy * 0.5);
                //ankle pitch axis
                sio->servo_data[20] = sio->servo_data[20] - (int)(sio->gain_gx * gx);
                //ankle roll axis
                sio->servo_data[21]  = sio->servo_data[21] + (int)(sio->gain_gy * gy);
            }

            //body yaw axis
            sio->servo_data[11]  = sio->servo_data[11] + (int)(sio->gain_gz * gz);
        }

        //servo data send to servo
        id_R = 11;
        for (id_L = 0; id_L < 11; id_L++) {
            move_servo(&servo_R, id_L, sio->servo_data[id_L], &return_data);
            move_servo(&servo_R, id_R, sio->servo_data[id_R], &return_data);
            id_R++;
        }
    }

    if (command == 'c') {
        sub_command = pc.getc();
        if(sub_command == '0') {
            sio->motion_log_flg = 0;
        }
        if(sub_command == '1') {
            sio->motion_log_datanum = 0;
            sio->motion_log_flg = 1;
        }
        if(sub_command == '2') {
            //Send back ack
            while (!pc.writeable());
            for(i = 0; i < sio->motion_log_datanum; i++) {
                for(j = 0; j < 3; j++) {
                    pc.printf("%5.1f,", sio->motion_log[i][j]);
                }
                for(j = 3; j < 6; j++) {
                    pc.printf("%5.3f,", sio->motion_log[i][j]);
                }
                pc.printf("\n");
            }
            pc.putc(0x1a);
        }
    }

    //Battery voltage report
    if (command == 'M') {
        while (!pc.writeable());
        sio->Battery = Battery * 3.3 * 6.0;
        pc.printf("%5.1f\r\n",sio->Battery);
    }

    //Functional evaluation
    if (command == '9') {
    }
}

//*******************************************
//*******************************************
unsigned int read_ii(Serial *port)
{
    unsigned int    data;
    unsigned int    data_L, data_H;

    while (!port->readable());
    data_H = port->getc();

    while (!port->readable());
    data_L = port->getc();

    data = data_H * 256 + data_L;
    return(data);
}

//*******************************************
//*******************************************
void print_ii(Serial *port, unsigned int data)
{
    unsigned int    ii;
    unsigned char    data_L, data_H;

    if (data > 255) {
        ii = data / 256;
        data_H = (unsigned char)ii;
    } else {
        ii = 0;
        data_H = 0;
    }

    ii = data - ii * 256;
    data_L = (unsigned char)ii;

    while (!port->writeable());
    port->putc(data_H);

    while (!port->writeable());
    port->putc(data_L);

    return;
}
